@@ -1,9 +1,5 @@
-from __future__ import absolute_import
-
 import collections
 
-__metaclass__ = type
-
 
 class FeatureSet:
     """
