@@ -1,12 +1,9 @@
 import abc
 
-import six
-
 from tempora import schedule
 
 
-@six.add_metaclass(abc.ABCMeta)
-class IScheduler(object):
+class IScheduler(metaclass=abc.ABCMeta):
     @abc.abstractmethod
     def execute_every(self, period, func):
         "execute func every period"
